=== modified file 'NEWS'
--- NEWS	2005-11-30 06:41:00 +0000
+++ NEWS	2005-11-30 15:35:12 +0000
@@ -23,6 +23,14 @@
       This gives better integration with user settings such as ProxyCommand.
       (James Henstridge)
 
+    * Sftp paths can now be relative, or local, according to the IETF draft spec.
+      Paths now take the form:
+      sftp://user:pass@host:port/relative/path
+      or
+      sftp://user:pass@host:port/%2Fabsolute/path
+      or
+      sftp://user:pass@host:port//absolute/path
+
   IMPROVEMENTS:
 
     * "bzr INIT dir" now initializes the specified directory, and creates 
@@ -64,6 +72,10 @@
     * Improved performance when integrating changes from a remote weave.  
       (Goffredo Baroncelli)
 
+    * Sftp will attempt to cache the connection, so it is more likely that
+      a connection will be reused, rather than requiring multiple password
+      requests.
+
   BUG FIXES:
 
     * SFTP can walk up to the root path without index errors. (Robert Collins)

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-11-23 15:44:24 +0000
+++ bzrlib/__init__.py	2005-11-30 14:55:03 +0000
@@ -79,7 +79,14 @@
 IGNORE_FILENAME = ".bzrignore"
 
 import os
-import locale
+import sys
+if sys.platform == 'darwin':
+    # work around egregious python 2.4 bug
+    sys.platform = 'posix'
+    import locale
+    sys.platform = 'darwin'
+else:
+    import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-11-30 00:25:50 +0000
+++ bzrlib/builtins.py	2005-11-30 15:15:17 +0000
@@ -395,7 +395,8 @@
                                   "  Try merge.")
         if br_to.get_parent() is None or remember:
             br_to.set_parent(location)
-        note('%d revision(s) pulled.', count)
+        note('%d revision(s) pulled.' % (count,))
+
         if verbose:
             new_rh = tree_to.branch.revision_history()
             if old_rh != new_rh:
@@ -486,6 +487,7 @@
         if br_from.get_push_location() is None or remember:
             br_from.set_push_location(location)
         note('%d revision(s) pushed.' % (count,))
+
         if verbose:
             new_rh = br_to.revision_history()
             if old_rh != new_rh:
@@ -1271,7 +1273,7 @@
             raise BzrCommandError("Commit refused because there are unknown "
                                   "files in the working tree.")
         note('Committed revision %d.' % (tree.branch.revno(),))
-        
+
 
 class cmd_check(Command):
     """Validate consistency of branch history.

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-11-23 06:25:50 +0000
+++ bzrlib/commands.py	2005-11-30 15:43:57 +0000
@@ -543,9 +543,10 @@
 
 def main(argv):
     import bzrlib.ui
+    from bzrlib.ui.text import TextUIFactory
     ## bzrlib.trace.enable_default_logging()
     bzrlib.trace.log_startup(argv)
-    bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
+    bzrlib.ui.ui_factory = TextUIFactory()
     ret = run_bzr_catch_errors(argv[1:])
     mutter("return code %d", ret)
     return ret

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-11-29 01:20:57 +0000
+++ bzrlib/osutils.py	2005-11-30 14:55:03 +0000
@@ -240,7 +240,12 @@
 
 def pumpfile(fromfile, tofile):
     """Copy contents of one file to another."""
-    tofile.write(fromfile.read())
+    BUFSIZE = 32768
+    while True:
+        b = fromfile.read(BUFSIZE)
+        if not b:
+            break
+        tofile.write(b)
 
 
 def sha_file(f):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-11-29 22:45:08 +0000
+++ bzrlib/tests/__init__.py	2005-11-30 16:41:08 +0000
@@ -653,6 +653,7 @@
                    'bzrlib.tests.test_workingtree',
                    'bzrlib.tests.test_upgrade',
                    'bzrlib.tests.test_uncommit',
+                   'bzrlib.tests.test_ui',
                    'bzrlib.tests.test_conflicts',
                    'bzrlib.tests.test_testament',
                    'bzrlib.tests.test_annotate',

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2005-10-27 03:28:48 +0000
+++ bzrlib/tests/stub_sftp.py	2005-11-20 17:00:43 +0000
@@ -25,11 +25,17 @@
 
 
 class StubServer (ServerInterface):
+    def __init__(self, test_case):
+        ServerInterface.__init__(self)
+        self._test_case = test_case
+
     def check_auth_password(self, username, password):
         # all are allowed
+        self._test_case.log('sftpserver - authorizing: %s' % (username,))
         return AUTH_SUCCESSFUL
 
     def check_channel_request(self, kind, chanid):
+        self._test_case.log('sftpserver - channel request: %s, %s' % (kind, chanid))
         return OPEN_SUCCEEDED
 
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2005-11-23 15:44:24 +0000
+++ bzrlib/tests/test_http.py	2005-11-30 15:15:17 +0000
@@ -1,9 +1,27 @@
 # (C) 2005 Canonical
 
 from bzrlib.tests import TestCase
-from bzrlib.transport.http import HttpTransport
-
+from bzrlib.transport.http import HttpTransport, extract_auth
+
+class FakeManager (object):
+    def __init__(self):
+        self.credentials = []
+        
+    def add_password(self, realm, host, username, password):
+        self.credentials.append([realm, host, username, password])
+
+        
 class TestHttpUrls(TestCase):
+    def test_url_parsing(self):
+        f = FakeManager()
+        url = extract_auth('http://example.com', f)
+        self.assertEquals('http://example.com', url)
+        self.assertEquals(0, len(f.credentials))
+        url = extract_auth('http://user:pass@www.bazaar-ng.org/bzr/bzr.dev', f)
+        self.assertEquals('http://www.bazaar-ng.org/bzr/bzr.dev', url)
+        self.assertEquals(1, len(f.credentials))
+        self.assertEquals([None, 'www.bazaar-ng.org', 'user', 'pass'], f.credentials[0])
+        
     def test_abs_url(self):
         """Construction of absolute http URLs"""
         t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')

=== modified file 'bzrlib/tests/test_sftp.py'
--- bzrlib/tests/test_sftp.py	2005-11-30 06:41:00 +0000
+++ bzrlib/tests/test_sftp.py	2005-11-30 15:32:20 +0000
@@ -17,9 +17,8 @@
 import os
 import socket
 import threading
-import unittest
 
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.tests.test_transport import TestTransportMixIn
 
 try:
@@ -74,6 +73,11 @@
     
     def stop(self):
         self.stop_event.set()
+        # We should consider waiting for the other thread
+        # to stop, because otherwise we get spurious
+        #   bzr: ERROR: Socket exception: Connection reset by peer (54)
+        # because the test suite finishes before the thread has a chance
+        # to close. (Especially when only running a few tests)
         
         
 class TestCaseWithSFTPServer (TestCaseInTempDir):
@@ -88,7 +92,7 @@
         file(key_file, 'w').write(STUB_SERVER_KEY)
         host_key = paramiko.RSAKey.from_private_key_file(key_file)
         ssh_server.add_server_key(host_key)
-        server = StubServer()
+        server = StubServer(self)
         ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer, StubSFTPServer, root=self._root)
         event = threading.Event()
         ssh_server.start_server(event, server)
@@ -98,15 +102,19 @@
     def setUp(self):
         TestCaseInTempDir.setUp(self)
         self._root = self.test_dir
+        self._is_setup = False
 
     def delayed_setup(self):
         # some tests are just stubs that call setUp and then immediately call
         # tearDwon.  so don't create the port listener until get_transport is
         # called and we know we're in an actual test.
+        if self._is_setup:
+            return
         self._listener = SingleListener(self._run_server)
         self._listener.setDaemon(True)
         self._listener.start()        
         self._sftp_url = 'sftp://foo:bar@localhost:%d/' % (self._listener.port,)
+        self._is_setup = True
         
     def tearDown(self):
         try:
@@ -118,12 +126,19 @@
         
 class SFTPTransportTest (TestCaseWithSFTPServer, TestTransportMixIn):
     readonly = False
-    setup = True
+
+    def setUp(self):
+        TestCaseWithSFTPServer.setUp(self)
+        self.sftplogs = []
+
+    def log(self, *args):
+        """Override the default log to grab sftp server messages"""
+        TestCaseWithSFTPServer.log(self, *args)
+        if args and args[0].startswith('sftpserver'):
+            self.sftplogs.append(args[0])
 
     def get_transport(self):
-        if self.setup:
-            self.delayed_setup()
-            self.setup = False
+        self.delayed_setup()
         from bzrlib.transport.sftp import SFTPTransport
         url = self._sftp_url
         return SFTPTransport(url)
@@ -153,20 +168,34 @@
         l.unlock()
         l2.unlock()
 
+    def test_multiple_connections(self):
+        t = self.get_transport()
+        self.assertEquals(self.sftplogs, 
+                ['sftpserver - authorizing: foo'
+               , 'sftpserver - channel request: session, 1'])
+        self.sftplogs = []
+        # The second request should reuse the first connection
+        # SingleListener only allows for a single connection,
+        # So the next line fails unless the connection is reused
+        t2 = self.get_transport()
+        self.assertEquals(self.sftplogs, [])
+
 
 class FakeSFTPTransport (object):
     _sftp = object()
 fake = FakeSFTPTransport()
 
 
-class SFTPNonServerTest(unittest.TestCase):
+class SFTPNonServerTest(TestCase):
     def test_parse_url(self):
         from bzrlib.transport.sftp import SFTPTransport
         s = SFTPTransport('sftp://simple.example.com/%2fhome/source', clone_from=fake)
         self.assertEquals(s._host, 'simple.example.com')
-        self.assertEquals(s._port, None)
+        self.assertEquals(s._port, 22)
         self.assertEquals(s._path, '/home/source')
-        self.assert_(s._password is None)
+        self.failUnless(s._password is None)
+
+        self.assertEquals(s.base, 'sftp://simple.example.com/%2Fhome/source')
         
         s = SFTPTransport('sftp://ro%62ey:h%40t@example.com:2222/relative', clone_from=fake)
         self.assertEquals(s._host, 'example.com')
@@ -175,6 +204,20 @@
         self.assertEquals(s._password, 'h@t')
         self.assertEquals(s._path, 'relative')
 
+        # Base should not keep track of the password
+        self.assertEquals(s.base, 'sftp://robey@example.com:2222/relative')
+
+        # Double slash should be accepted instead of using %2F
+        s = SFTPTransport('sftp://user@example.com:22//absolute/path', clone_from=fake)
+        self.assertEquals(s._host, 'example.com')
+        self.assertEquals(s._port, 22)
+        self.assertEquals(s._username, 'user')
+        self.assertEquals(s._password, None)
+        self.assertEquals(s._path, '/absolute/path')
+
+        # Also, don't show the port if it is the default 22
+        self.assertEquals(s.base, 'sftp://user@example.com/%2Fabsolute/path')
+
     def test_parse_invalid_url(self):
         from bzrlib.transport.sftp import SFTPTransport, SFTPTransportError
         try:
@@ -185,7 +228,6 @@
             self.assertEquals(str(e), 
                     '~janneke: invalid port number')
 
-        
 
 class SFTPBranchTest(TestCaseWithSFTPServer):
     """Test some stuff when accessing a bzr Branch over sftp"""

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_transport.py	2005-11-30 14:55:03 +0000
@@ -458,11 +458,15 @@
 
     def test_delete(self):
         # TODO: Test Transport.delete
-        pass
+        t = self.get_transport()
 
     def test_move(self):
         # TODO: Test Transport.move
-        pass
+        t = self.get_transport()
+
+    def test_copy(self):
+        # TODO: Test Transport.move
+        t = self.get_transport()
 
     def test_connection_error(self):
         """ConnectionError is raised when connection is impossible"""

=== added file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_ui.py	2005-11-30 16:41:08 +0000
@@ -0,0 +1,52 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the bzrlib ui
+"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCase
+from bzrlib.ui import SilentUIFactory
+from bzrlib.ui.text import TextUIFactory
+
+class UITests(TestCase):
+
+    def test_silent_factory(self):
+
+        ui = SilentUIFactory()
+        pb = ui.progress_bar()
+        # TODO: Test that there is no output from SilentUIFactory
+
+        self.assertEquals(ui.get_password(), None)
+        self.assertEquals(ui.get_password(u'Hello There \u1234 %(user)s',
+                                          user=u'some\u1234')
+                         , None)
+
+    def test_text_factory(self):
+        ui = TextUIFactory()
+        pb = ui.progress_bar()
+        # TODO: Test the output from TextUIFactory, perhaps by overriding sys.stdout
+
+        # Unfortunately we can't actually test the ui.get_password() because 
+        # that would actually prompt the user for a password during the test suite
+        # This has been tested manually with both LANG=en_US.utf-8 and LANG=C
+        # print
+        # self.assertEquals(ui.get_password(u"%(user)s please type 'bogus'",
+        #                                   user=u'some\u1234')
+        #                  , 'bogus')
+

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-11-29 07:58:36 +0000
+++ bzrlib/trace.py	2005-11-30 15:15:17 +0000
@@ -54,6 +54,7 @@
 _trace_file = None
 _bzr_log_file = None
 
+
 class QuietFormatter(logging.Formatter):
     """Formatter that supresses the details of errors.
 

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-11-23 06:25:50 +0000
+++ bzrlib/transport/http.py	2005-11-30 15:15:17 +0000
@@ -21,7 +21,7 @@
                            NonRelativePath, TransportError, ConnectionError)
 import os, errno
 from cStringIO import StringIO
-import urllib2
+import urllib, urllib2
 import urlparse
 
 from bzrlib.errors import BzrError, BzrCheckError
@@ -29,10 +29,45 @@
 from bzrlib.trace import mutter
 
 
+def extract_auth(url, password_manager):
+    """
+    Extract auth parameters from am HTTP/HTTPS url and add them to the given
+    password manager.  Return the url, minus those auth parameters (which
+    confuse urllib2).
+    """
+    assert url.startswith('http://') or url.startswith('https://')
+    scheme, host = url.split('//', 1)
+    if '/' in host:
+        host, path = host.split('/', 1)
+        path = '/' + path
+    else:
+        path = ''
+    port = ''
+    if '@' in host:
+        auth, host = host.split('@', 1)
+        if ':' in auth:
+            username, password = auth.split(':', 1)
+        else:
+            username, password = auth, None
+        if ':' in host:
+            host, port = host.split(':', 1)
+            port = ':' + port
+        # FIXME: if password isn't given, should we ask for it?
+        if password is not None:
+            username = urllib.unquote(username)
+            password = urllib.unquote(password)
+            password_manager.add_password(None, host, username, password)
+    url = scheme + '//' + host + port + path
+    return url
+    
 def get_url(url):
     import urllib2
-    mutter("get_url %s", url)
-    url_f = urllib2.urlopen(url)
+    mutter("get_url %s" % url)
+    manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+    url = extract_auth(url, manager)
+    auth_handler = urllib2.HTTPBasicAuthHandler(manager)
+    opener = urllib2.build_opener(auth_handler)
+    url_f = opener.open(url)
     return url_f
 
 class HttpTransportError(TransportError):

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-11-30 06:41:00 +0000
+++ bzrlib/transport/sftp.py	2005-11-30 16:41:08 +0000
@@ -27,6 +27,7 @@
 import time
 import random
 import subprocess
+import weakref
 
 from bzrlib.errors import (FileExists, 
                            TransportNotPossible, NoSuchFile, NonRelativePath,
@@ -35,6 +36,7 @@
 from bzrlib.config import config_dir
 from bzrlib.trace import mutter, warning, error
 from bzrlib.transport import Transport, register_transport
+from bzrlib.ui import ui_factory
 
 try:
     import paramiko
@@ -128,10 +130,15 @@
         self.proc.wait()
 
 
-
 SYSTEM_HOSTKEYS = {}
 BZR_HOSTKEYS = {}
 
+# This is a weakref dictionary, so that we can reuse connections
+# that are still active. Long term, it might be nice to have some
+# sort of expiration policy, such as disconnect if inactive for
+# X seconds. But that requires a lot more fanciness.
+_connected_hosts = weakref.WeakValueDictionary()
+
 def load_host_keys():
     """
     Load system host keys (probably doesn't work on windows) and any
@@ -208,11 +215,13 @@
     """
     Transport implementation for SFTP access.
     """
+    _do_prefetch = False # Right now Paramiko's prefetch support causes things to hang
 
     def __init__(self, base, clone_from=None):
         assert base.startswith('sftp://')
+        self._parse_url(base)
+        base = self._unparse_url()
         super(SFTPTransport, self).__init__(base)
-        self._parse_url(base)
         if clone_from is None:
             self._sftp_connect()
         else:
@@ -299,11 +308,8 @@
         try:
             path = self._abspath(relpath)
             f = self._sftp.file(path)
-            try:
+            if self._do_prefetch and hasattr(f, 'prefetch'):
                 f.prefetch()
-            except AttributeError:
-                # only works on paramiko 1.5.1 or greater
-                pass
             return f
         except (IOError, paramiko.SSHException), x:
             raise NoSuchFile('Error retrieving %s: %s' % (path, str(x)), x)
@@ -323,11 +329,8 @@
         # TODO: implement get_partial_multi to help with knit support
         f = self.get(relpath)
         f.seek(start)
-        try:
+        if self._do_prefetch and hasattr(f, 'prefetch'):
             f.prefetch()
-        except AttributeError:
-            # only works on paramiko 1.5.1 or greater
-            pass
         return f
 
     def put(self, relpath, f):
@@ -369,15 +372,22 @@
                 file_existed = True
             except:
                 file_existed = False
+            success = False
             try:
-                self._sftp.rename(tmp_abspath, final_path)
-            except IOError, e:
-                self._translate_io_exception(e, relpath)
-            except paramiko.SSHException, x:
-                raise SFTPTransportError('Unable to rename into file %r' 
-                                          % (relpath,), x)
-            if file_existed:
-                self._sftp.unlink(tmp_safety)
+                try:
+                    self._sftp.rename(tmp_abspath, final_path)
+                except IOError, e:
+                    self._translate_io_exception(e, relpath)
+                except paramiko.SSHException, x:
+                    raise SFTPTransportError('Unable to rename into file %r' % (path,), x) 
+                else:
+                    success = True
+            finally:
+                if file_existed:
+                    if success:
+                        self._sftp.unlink(tmp_safety)
+                    else:
+                        self._sftp.rename(tmp_safety, final_path)
 
     def iter_files_recursive(self):
         """Walk the relative paths of all files in this transport."""
@@ -430,6 +440,20 @@
         """Copy the item at rel_from to the location at rel_to"""
         path_from = self._abspath(rel_from)
         path_to = self._abspath(rel_to)
+        self._copy_abspaths(path_from, path_to)
+
+    def _copy_abspaths(self, path_from, path_to):
+        """Copy files given an absolute path
+
+        :param path_from: Path on remote server to read
+        :param path_to: Path on remote server to write
+        :return: None
+
+        TODO: Should the destination location be atomically created?
+              This has not been specified
+        TODO: This should use some sort of remote copy, rather than
+              pulling the data locally, and then writing it remotely
+        """
         try:
             fin = self._sftp.file(path_from, 'rb')
             try:
@@ -444,6 +468,33 @@
         except (IOError, paramiko.SSHException), x:
             raise SFTPTransportError('Unable to copy %r to %r' % (path_from, path_to), x)
 
+    def copy_to(self, relpaths, other, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        """
+        if isinstance(other, SFTPTransport) and other._sftp is self._sftp:
+            # Both from & to are on the same remote filesystem
+            # We can use a remote copy, instead of pulling locally, and pushing
+
+            total = self._get_total(relpaths)
+            count = 0
+            for path in relpaths:
+                path_from = self._abspath(relpath)
+                path_to = other._abspath(relpath)
+                self._update_pb(pb, 'copy-to', count, total)
+                self._copy_abspaths(path_from, path_to)
+                count += 1
+            return count
+        else:
+            return super(SFTPTransport, self).copy_to(relpaths, other, pb=pb)
+
+        # The dummy implementation just does a simple get + put
+        def copy_entry(path):
+            other.put(path, self.get(path))
+
+        return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
+
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""
         path_from = self._abspath(rel_from)
@@ -522,7 +573,7 @@
         netloc = urllib.quote(self._host)
         if self._username is not None:
             netloc = '%s@%s' % (urllib.quote(self._username), netloc)
-        if self._port is not None:
+        if self._port not in (None, 22):
             netloc = '%s:%d' % (netloc, self._port)
 
         return urlparse.urlunparse(('sftp', netloc, path, '', '', ''))
@@ -565,27 +616,41 @@
     def _parse_url(self, url):
         (self._username, self._password,
          self._host, self._port, self._path) = self._split_url(url)
+        if self._port is None:
+            self._port = 22
 
     def _sftp_connect(self):
+        """Connect to the remote sftp server.
+        After this, self._sftp should have a valid connection (or
+        we raise an SFTPTransportError 'could not connect').
+
+        TODO: Raise a more reasonable ConnectionFailed exception
+        """
+        global _connected_hosts
+
+        idx = (self._host, self._port, self._username)
+        try:
+            self._sftp = _connected_hosts[idx]
+            return
+        except KeyError:
+            pass
+        
         vendor = _get_ssh_vendor()
-        if (self._path is None) or (self._path == ''):
-            self._path = ''
-        else:
-            # remove leading '/'
-            self._path = urllib.unquote(self._path[1:])
         if vendor != 'none':
             sock = SFTPSubprocess(self._host, self._port, self._username)
             self._sftp = SFTPClient(sock)
         else:
             self._paramiko_connect()
 
+        _connected_hosts[idx] = self._sftp
+
     def _paramiko_connect(self):
         global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
         
         load_host_keys()
 
         try:
-            t = paramiko.Transport((self._host, self._port or 22))
+            t = paramiko.Transport((self._host, self._port))
             t.start_client()
         except paramiko.SSHException:
             raise SFTPTransportError('Unable to reach SSH host %s:%d' % (self._host, self._port))
@@ -614,7 +679,7 @@
                 (self._host, our_server_key_hex, server_key_hex),
                 ['Try editing %s or %s' % (filename1, filename2)])
 
-        self._sftp_auth(t, self._username or getpass.getuser(), self._host)
+        self._sftp_auth(t)
         
         try:
             self._sftp = t.open_sftp_client()
@@ -622,54 +687,62 @@
             raise BzrError('Unable to find path %s on SFTP server %s' % \
                 (self._path, self._host))
 
-    def _sftp_auth(self, transport, username, host):
+    def _sftp_auth(self, transport):
+        # paramiko requires a username, but it might be none if nothing was supplied
+        # use the local username, just in case.
+        # We don't override self._username, because if we aren't using paramiko,
+        # the username might be specified in ~/.ssh/config and we don't want to
+        # force it to something else
+        # Also, it would mess up the self.relpath() functionality
+        username = self._username or getpass.getuser()
+
         agent = paramiko.Agent()
         for key in agent.get_keys():
             mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
             try:
-                transport.auth_publickey(self._username, key)
+                transport.auth_publickey(username, key)
                 return
             except paramiko.SSHException, e:
                 pass
         
         # okay, try finding id_rsa or id_dss?  (posix only)
-        if self._try_pkey_auth(transport, paramiko.RSAKey, 'id_rsa'):
-            return
-        if self._try_pkey_auth(transport, paramiko.DSSKey, 'id_dsa'):
-            return
+        if self._try_pkey_auth(transport, paramiko.RSAKey, username, 'id_rsa'):
+            return
+        if self._try_pkey_auth(transport, paramiko.DSSKey, username, 'id_dsa'):
+            return
+
 
         if self._password:
             try:
-                transport.auth_password(self._username, self._password)
+                transport.auth_password(username, self._password)
                 return
             except paramiko.SSHException, e:
                 pass
 
+            # FIXME: Don't keep a password held in memory if you can help it
+            #self._password = None
+
         # give up and ask for a password
-        # FIXME: shouldn't be implementing UI this deep into bzrlib
-        enc = sys.stdout.encoding
-        password = getpass.getpass('SSH %s@%s password: ' %
-            (self._username.encode(enc, 'replace'), self._host.encode(enc, 'replace')))
+        password = ui_factory.get_password(prompt='SSH %(user)s@%(host)s password',
+                                           user=username, host=self._host)
         try:
-            transport.auth_password(self._username, password)
+            transport.auth_password(username, password)
         except paramiko.SSHException:
             raise SFTPTransportError('Unable to authenticate to SSH host as %s@%s' % \
-                (self._username, self._host))
+                (username, self._host))
 
-    def _try_pkey_auth(self, transport, pkey_class, filename):
+    def _try_pkey_auth(self, transport, pkey_class, username, filename):
         filename = os.path.expanduser('~/.ssh/' + filename)
         try:
             key = pkey_class.from_private_key_file(filename)
-            transport.auth_publickey(self._username, key)
+            transport.auth_publickey(username, key)
             return True
         except paramiko.PasswordRequiredException:
-            # FIXME: shouldn't be implementing UI this deep into bzrlib
-            enc = sys.stdout.encoding
-            password = getpass.getpass('SSH %s password: ' % 
-                (os.path.basename(filename).encode(enc, 'replace'),))
+            password = ui_factory.get_password(prompt='SSH %(filename)s password',
+                                               filename=filename)
             try:
                 key = pkey_class.from_private_key_file(filename, password)
-                transport.auth_publickey(self._username, key)
+                transport.auth_publickey(username, key)
                 return True
             except paramiko.SSHException:
                 mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
@@ -692,7 +765,7 @@
 
         :param relpath: The relative path, where the file should be opened
         """
-        path = self._abspath(relpath)
+        path = self._sftp._adjust_cwd(self._abspath(relpath))
         attr = SFTPAttributes()
         mode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
                 | SFTP_FLAG_TRUNC | SFTP_FLAG_EXCL)

=== added directory 'bzrlib/ui'
=== renamed file 'bzrlib/ui.py' => 'bzrlib/ui/__init__.py'
--- bzrlib/ui.py	2005-09-19 06:05:19 +0000
+++ bzrlib/ui/__init__.py	2005-11-30 16:41:08 +0000
@@ -28,24 +28,42 @@
 displays no output.
 """
 
-
-
-
-
 import bzrlib.progress
 
 
-class TextUIFactory(object):
+class UIFactory(object):
+    """UI abstraction.
+
+    This tells the library how to display things to the user.  Through this
+    layer different applications can choose the style of UI.
+    """
     def progress_bar(self):
-
-        # this in turn is abstract, and creates either a tty or dots
-        # bar depending on what we think of the terminal
-        return bzrlib.progress.ProgressBar()
-
-
-class SilentUIFactory(object):
+        """Return a progress bar object"""
+        raise NotImplementedError
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        raise NotImplementedError
+        
+
+class SilentUIFactory(UIFactory):
+    """A UI Factory which never prints anything.
+
+    This is the default UI, if another one is never registered.
+    """
     def progress_bar(self):
         return bzrlib.progress.DummyProgress()
 
+    def get_password(self, prompt='', **kwargs):
+        return None
+
 
 ui_factory = SilentUIFactory()

=== added file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui/text.py	2005-11-30 16:41:08 +0000
@@ -0,0 +1,52 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""Text UI, write output to the console.
+"""
+
+import getpass
+import sys
+
+import bzrlib.progress
+from bzrlib.ui import UIFactory
+
+
+class TextUIFactory(UIFactory):
+    def progress_bar(self):
+
+        # this in turn is abstract, and creates either a tty or dots
+        # bar depending on what we think of the terminal
+        return bzrlib.progress.ProgressBar()
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        prompt = (prompt % kwargs).encode(sys.stdout.encoding, 'replace')
+        prompt += ': '
+        try:
+            return getpass.getpass(prompt)
+        except KeyboardInterrupt:
+            return None
+

